package ru.volgait.task4;

import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.StringJoiner;

/**
 * Created by Mikhail Tarapata on 29.04.2017.
 */
public class JsonSerializerImpl implements JsonSerializer {

    public static final String NULL = "null";

    @Override
    public String serialize(Object obj) throws InvocationTargetException, IllegalAccessException {
        if (obj == null) {
            throw new IllegalArgumentException("Serializeble object can't be null");
        }

        StringBuilder result = new StringBuilder();

        if (obj instanceof Serializable) {
            result.append("\"");
            result.append(obj.toString());
            result.append("\",");
            return result.toString();
        }

        result.append("{");

        Method[] methods = obj.getClass().getMethods();
        for (Method method : methods) {
            if (isGetter(method)) {
                if (isArrayReturned(method)) {

                    String methodName = method.getName();
                    result.append("\"");
                    result.append(methodName.substring(3, 4).toLowerCase());
                    result.append(methodName.substring(4, method.getName().length()));
                    result.append("\" : [");
                    Object[] array = (Object[]) method.invoke(obj, null);
                    if (array == null) {
                        result.append(NULL);
                    } else {
                        for (Object o : array) {
                            result.append(serialize(o));
                        }
                    }

                    result.append("],");
                } else {
                    if (method.getReturnType().isPrimitive()) {
                        Object res = buildJsonField(obj, result, method);
                        result.append("\" : \"" + res + "\",");
                    } else {
                        Object res = buildJsonField(obj, result, method);
                        result.append("\" : " + serialize(res));
                    }
                }
            }
        }

        result.append("}");
        return result.toString();
    }

    private Object buildJsonField(Object obj, StringBuilder result, Method method) throws IllegalAccessException, InvocationTargetException {
        String methodName = method.getName();
        result.append("\"");
        result.append(methodName.substring(3, 4).toLowerCase());
        result.append(methodName.substring(4, method.getName().length()));
        return method.invoke(obj, null);
    }

    private boolean isGetter(Method method) {
        boolean result = true;
        if (!method.getName().startsWith("get") || method.getName().equals("getClass")) {
            result = false;
        }
        if (method.getParameterTypes().length != 0) {
            result = false;
        }
        if (void.class.equals(method.getReturnType())) {
            result = false;
        }
        return result;
    }

    private boolean isArrayReturned(Method method) {
        return method.getReturnType().isArray();
    }

    private StringBuilder buildJsonField(String fieldName, String value) {
        StringBuilder name = new StringBuilder();
        name.append("\"");
        name.append(fieldName);
        name.append("\n : \"");
        name.append(value);
        name.append("\"");
        return name;
    }
}